function angle_check()
% -- Function File: angle_check()
%     Checks for every attitude with roll, pitch, yaw in the range
%     -180 < alpha <= 180 if the inversely calculated tilt and heading is
%     correct.
%
%     Arnold Neugebauer

page_screen_output( 0 )      %don't use pager
page_output_immediately( 1 ) %immediately display (flash) output

stepsize = 15; %degrees

acc_strength = 18;
acc_g = [ 0 0 acc_strength ]; %normal vectors here, plotvector is not used

mfield_strength = 18;
mfield_inclination = 0; -70;     %northern europe would be about +70 degrees
mfieldx_g = mfield_strength * cosd( mfield_inclination );
mfieldy_g = 0;
mfieldz_g = sqrt( mfield_strength^2 - mfieldx_g^2 );
if( mfield_inclination > 0 )
   mfieldz_g = -mfieldz_g;
endif
mfield_g  = [ mfieldx_g, mfieldy_g, mfieldz_g ];

roll     = -180:stepsize:180;
roll(1)  = -180 + 1e-10; %-180 is ambiguous, the range is always -180 < alpha <= 180

pitch    = -180:stepsize:180;
pitch(1) = -180 + 1e-10;

yaw      = -180:stepsize:180;
yaw(1)   = -180 + 1e-10;

error_flag = 0;
progress_start = time();
printf( "." )

for r_idx = 1:length(roll)
   rxn = rotx( -roll(r_idx) );

   for p_idx = 1:length(pitch)
      ryn = roty( -pitch(p_idx) );

      for y_idx = 1:length(yaw)
         rzn = rotz( -yaw(y_idx) );
         rn  = rzn * ryn * rxn;

         %-------------------display progress-----------------------------------
         progress_current = time();
         if( (progress_current - progress_start) > 3 )
            progress_start = progress_current;
            printf( "." )
         endif

         %-------------------measurement----------------------------------------
         acc_v  = acc_g * rn;
         accx   = acc_v(1);
         accy   = acc_v(2);
         accz   = acc_v(3);

         magn_v = mfield_g * rn;
         magnx  = magn_v(1);
         magny  = magn_v(2);
         magnz  = magn_v(3);

         %-------------------tilt and heading-----------------------------------
         if( accz == 0 ) %discard measurement to avoid division by 0
            break        %(pitch and/or roll = 90 degrees)
         endif

         roll1  = atan2d( accy, accz );
         pitch1 = - atand( accx / accz * cosd(roll1) );

         magnxc = magnx * cosd( pitch1 ) + magny * sind( roll1 ) * sind( pitch1 ) + magnz * cosd( roll1 ) * sind( pitch1 );
         magnyc = magny * cosd( roll1 )  - magnz * sind( roll1 );
         yaw1 = atan2d( - magnyc, magnxc ); %heading

         %-------------------check----------------------------------------------
         %calculate expected result from given attitude
         roll0  = roll(r_idx);
         pitch0 = pitch(p_idx);
         yaw0   = yaw(y_idx);

         if( abs(pitch0) > 90 )
            roll0 += 180;     %flip
            if( roll0 > 180 ) %pull into standard range
               roll0 -= 360;
            endif

            pitch0 += 180;
            if( pitch0 > 180 )
               pitch0 -= 360;
            endif
            pitch0 = -pitch0;     %invert pitch

            yaw0 += 180;
            if( yaw0 > 180 )
               yaw0 -= 360;
            endif
         endif

         %Move point of discontinuity (+-180 degrees) for comparison of angles
         if( abs(roll0) > 90 )
            roll0 += 180;
            if( roll0 > 180 )
               roll0 -= 360;
            endif

            roll1 += 180;
            if( roll1 > 180 )
               roll1 -= 360;
            endif
         endif

         if( abs(yaw0) > 90 )
            yaw0 += 180;
            if( yaw0 > 180 )
               yaw0 -= 360;
            endif

            yaw1 += 180;
            if( yaw1 > 180 )
               yaw1 -= 360;
            endif
         endif

         total_error = abs( roll1 - roll0 ) + abs( pitch1 - pitch0 ) +  abs( yaw1 - yaw0 );
         if( total_error > 0.01 )
            error_flag = 1;
         endif

      endfor
   endfor
endfor

disp( "" );
if( error_flag == 0 )
   disp( "Angle check ok" );
else
   disp( "Angle check failed" );
endif

endfunction
