%implementation.m
% -- Script File: implementation
%     Simulates a possible implementation taking various error sources into
%     account.
%
%     Arnold Neugebauer

page_screen_output( 0 )       %don't use pager
page_output_immediately( 1 )  %immediately display (flash) output
clear


%todo: let acc point down (also in angle_check)
acc_strength = 1;             %g
acc_g = [ 0 0 acc_strength ]; %normal vectors here, plotvector is not used

mfield_strength = 1;          %e.g. 40 A/m, normalized to 1
mfield_inclination = 70;      %northern europe
mfieldx_g = mfield_strength * cosd( mfield_inclination );
mfieldy_g = 0;
mfieldz_g = sqrt( mfield_strength^2 - mfieldx_g^2 );
if( mfield_inclination > 0 )
   mfieldz_g = -mfieldz_g;
endif
mfield_g  = [ mfieldx_g, mfieldy_g, mfieldz_g ];


%Scaling and offset applied to the field components. Gives measurement values
%like hardware ADC samples.
%(magn offsets may be negative because flipping combines two ADC samples to one
%measurement value - for flipping see KMZ51 application note)
%Values must correspond with _c values below.
acc_scalex  = 143;     %LIS344 @ 3V3 50C above room temperature with part variation
acc_scaley  = 135;     %directly connected to a 10 bit ADC @ 5V
acc_scalez  = 127;     %(arbitrarily entered pos. or neg. deviations to typical values,
acc_offsetx = 334;     %estimated from performance figures of LIS344)
acc_offsety = 338;
acc_offsetz = 342;

magn_scalex  = 409;    %KMZ51 @ 5V exposed to 40 A/m, deviations estimated
magn_scaley  = 391;    %from sensitivity temp coefficient at 20K below and 50K
magn_scalez  = 372;    %above room temperature
magn_offsetx = -40;    %nearly equals _c offset values below, so I assume to get
magn_offsety = -162;   %rid of most offset errors by the flipping trick
magn_offsetz = -75;    %todo: add soft magnetic distortion here


%Scaling and offset applied to the measured ADC values to calculate back the
%field components. These values are usually determined by calibration.
%The _c values must correspond to but are usually not equal to the scaling and
%offset values above.
acc_scalex_c  = 135;   %LIS344 @ 3V3 and room temperature without part variation
acc_scaley_c  = 135;   %directly connected to a 10 bit ADC @ 5V
acc_scalez_c  = 135;
acc_offsetx_c = 338;
acc_offsety_c = 338;
acc_offsetz_c = 338;

acc_thres_c   = 1.1;   %g threshold for plausibilty check

magn_scalex_c  = 354;  %KMZ51 @ 5V exposed to 40 A/m @ room temperature,
magn_scaley_c  = 354;  %signal amplified by a faktor of 270
magn_scalez_c  = 354;
magn_offsetx_c = -48;  %actual offset values of my hardware
magn_offsety_c = -162;
magn_offsetz_c = -71;


%noise in percent of maximum measurement value (1.5% observed with my hardware
%on magn channels, acc channels are much more stable but here 1.5% is also
%applied to them)
noise_level = 1.5; %percent

acc_noise_levelx = acc_strength * acc_scalex * noise_level/100;
acc_noise_levely = acc_strength * acc_scaley * noise_level/100;
acc_noise_levelz = acc_strength * acc_scalez * noise_level/100;

magn_noise_levelx = mfield_strength * magn_scalex * noise_level/100;
magn_noise_levely = mfield_strength * magn_scaley * noise_level/100;
magn_noise_levelz = mfield_strength * magn_scalez * noise_level/100;


stepsize     = 15; %degrees
filter_depth = 10;


%larger angles for pitch generate the same sensor outputs twice, using them
%requires the code to translate between different combinations of roll, pitch
%and yaw describing the same attitude (see part2)
roll     = -180:stepsize:180;
roll(1)  = -180 + 1e-10; %-180 is ambiguous, the range is always -180 < alpha <= 180

pitch    = -90:stepsize:90;

yaw      = -180:stepsize:180;
yaw(1)   = -180 + 1e-10;


heading_error = zeros( size(pitch,2), size(roll,2) );
progress_start = time();
printf( "." )


for r_idx = 1:length(roll)
   rxn = rotx( -roll(r_idx) );

   for p_idx = 1:length(pitch)
      ryn = roty( -pitch(p_idx) );

      for y_idx = 1:length(yaw)
         rzn = rotz( -yaw(y_idx) );
         rn  = rzn * ryn * rxn;

         %-------------------display progress-----------------------------------
         progress_current = time();
         if( (progress_current - progress_start) > 3 )
            progress_start = progress_current;
            printf( "." )
         endif

         %-------------------measurement----------------------------------------
         %for clearity and better handling/performance with Octave all
         %measurements are collected in an array, in a typical implementation no
         %buffer is needed
         %todo: simulate clipping of acc sensor at 2g
         %      add vehicle acc to test plausibility check

         acc_v  = acc_g * rn;
         magn_v = mfield_g * rn;

         %add scaling and offset (translation between our real value and the
         %sensor output and/or a gain/offset error of the hardware and/or
         %magnetic material near the compass)
         accx_meas = acc_v(1) * acc_scalex + acc_offsetx;
         accy_meas = acc_v(2) * acc_scaley + acc_offsety;
         accz_meas = acc_v(3) * acc_scalez + acc_offsetz;

         magnx_meas = magn_v(1) * magn_scalex + magn_offsetx;
         magny_meas = magn_v(2) * magn_scaley + magn_offsety;
         magnz_meas = magn_v(3) * magn_scalez + magn_offsetz;

         %add some noise
         accx_meas += (rand( 1, filter_depth ) - 0.5) * acc_noise_levelx;
         accy_meas += (rand( 1, filter_depth ) - 0.5) * acc_noise_levely;
         accz_meas += (rand( 1, filter_depth ) - 0.5) * acc_noise_levelz;

         magnx_meas += (rand( 1, filter_depth ) - 0.5) * magn_noise_levelx;
         magny_meas += (rand( 1, filter_depth ) - 0.5) * magn_noise_levely;
         magnz_meas += (rand( 1, filter_depth ) - 0.5) * magn_noise_levelz;

         %-------------------noise filter---------------------------------------
         accx = sum( accx_meas ) / filter_depth;
         accy = sum( accy_meas ) / filter_depth;
         accz = sum( accz_meas ) / filter_depth;

         magnx = sum( magnx_meas ) / filter_depth;
         magny = sum( magny_meas ) / filter_depth;
         magnz = sum( magnz_meas ) / filter_depth;

         %-------------------offset/gain error compensation---------------------
         accx = (accx - acc_offsetx_c) / acc_scalex_c;
         accy = (accy - acc_offsety_c) / acc_scaley_c;
         accz = (accz - acc_offsetz_c) / acc_scalez_c;

         magnx = (magnx - magn_offsetx_c) / magn_scalex_c;
         magny = (magny - magn_offsety_c) / magn_scaley_c;
         magnz = (magnz - magn_offsetz_c) / magn_scalez_c;

         %since heading_error is initialized with 0 the following breaks lead to
         %an error of 0, canceled calculations are not counted as errors

         %-------------------discard measurements with accz = 0-----------------
         if( accz == 0 )
            break
         endif

         %-------------------plausibility check---------------------------------
         if( sqrt( accx^2 + accy^2 + accz^2 ) > acc_thres_c )
            break
         endif

         %-------------------tilt calculation-----------------------------------
         roll1  =   atan2( accy, accz ) * 180/pi;
         pitch1 = - atand( accx / accz * cosd(roll1) );

         %-------------------limit operational range----------------------------
         if( (abs(roll1) > 80) || (abs(pitch1) > 80) )
            break
         endif

         %-------------------heading calculation--------------------------------
         magnxc   = magnx * cosd( pitch1 ) + magny * sind( roll1 ) * sind( pitch1 ) + magnz * cosd( roll1 ) * sind( pitch1 );
         magnyc   = magny * cosd( roll1 )  - magnz * sind( roll1 );
         heading1 = atan2( - magnyc, magnxc ) * 180/pi;

         %-------------------check----------------------------------------------
         %Move point of discontinuity (+-180 degrees) for comparison of heading
         heading0 = yaw(y_idx);
         if( abs(heading0) > 90 )
            heading0 += 180;
            if( heading0 > 180 )
               heading0 -= 360;
            endif

            heading1 += 180;
            if( heading1 > 180 )
               heading1 -= 360;
            endif
         endif

         %if( abs(heading1 - heading0) > 10 )
         %   roll(r_idx)
         %   pitch(p_idx)
         %   yaw(y_idx)
         %   heading0
         %   heading1
         %   disp( "" );
         %endif

         %take the maximum error from all the calculations with different yaw
         heading_error(p_idx, r_idx) = max( heading_error(p_idx, r_idx), abs(heading1 - heading0) );

      endfor
   endfor
endfor

%maximum error from all the calculations with different yaw
mesh( roll, pitch, heading_error );
axis( [-80, 80, -80, 80, 0, 20] )
%axis( [0, 80, 0, 80, 0, 20] )
%axis( [-180, 180, -180, 180, 0, 20] ) %see all (but actually it is symmetrical)
title( "Heading Error" )
xlabel( "roll" )
ylabel( "pitch" )

disp( "" );
disp( "worst case heading error:" );
max( max(heading_error) )
