function plotaxes( rotation, length, char_scale, xy_rot_flag, color )
% -- Function File: plotaxes( rotation, length, color )
%     Plots the x, y and z axes of the coordinate system.
%     The rotation matrix defines the attitude in reference to the ground-fixed
%     system.
%     length is the total length of each axis. 1/3 of the length is plotted in
%     the negative, 2/3 in the positive direction.
%     The size of the XYZ characters is given by char_scale. char_scale = 1 is a
%     width of 1.2 and a height of 2.0.
%     xy_rot_flag is usually 0. xy_rot_flag = 1 applies an additional 90 degrees
%     rotation to X and Y, making them visible in views directly on the
%     xy-plane.
%     The position of the XYZ characters is beside axes for rotations near unity
%     and along the extension of axes otherwise.
%     The string 'color' is one of "k" (black), "r" (red), "g" (green),
%     "b" (blue), "m" (magenta), "c" (cyan), or "w" (white).
%
%     Arnold Neugebauer

%------------------------------------------------------------------------------
% plot axes
%------------------------------------------------------------------------------

lowpart = 1/3; %part of length below 0

lowlength  = -length * lowpart;
highlength =  length * (1 - lowpart);

vectorx = [lowlength 0 0; highlength 0 0];
vectory = [0 lowlength 0; 0 highlength 0];
vectorz = [0 0 lowlength; 0 0 highlength];

vectorx = vectorx * rotation;
vectory = vectory * rotation;
vectorz = vectorz * rotation;

oldhold = ishold();

plotvector( vectorx, color )
hold( "on" )
plotvector( vectory, color )
plotvector( vectorz, color )


%------------------------------------------------------------------------------
% plot characters
%------------------------------------------------------------------------------

char_width  = 1.2 * char_scale;
char_height = 2.0 * char_scale;

%the following is not an exact science...
char_disw = highlength - char_width;              %horizontal displacement
char_dish = highlength - char_height;             %vertical displacement
char_disd = sqrt( char_width^2 + char_height^2 ); %diagonal displacement

%'X' in the xz-plane
xchar_line1 = [-0.6, 0.0, 1.0; 0.6, 0.0,-1.0];
xchar_line2 = [-0.6, 0.0,-1.0; 0.6, 0.0, 1.0];

%'Y' in the yz-plane
ychar_line1 = [ 0.0,-0.6, 1.0; 0.0, 0.0, 0.0];
ychar_line2 = [ 0.0, 0.6, 1.0; 0.0, 0.0, 0.0];
ychar_line3 = [ 0.0, 0.0, 0.0; 0.0, 0.0,-1.0];

%'Z' in the yz-plane, 45 degrees rotated
zchar_line1 = [-0.42426, 0.42426, 1.0; 0.42426,-0.42426, 1.0];
zchar_line2 = [ 0.42426,-0.42426, 1.0;-0.42426, 0.42426,-1.0];
zchar_line3 = [-0.42426, 0.42426,-1.0; 0.42426,-0.42426,-1.0];

xchar_line1 *= char_scale;
xchar_line2 *= char_scale;

ychar_line1 *= char_scale;
ychar_line2 *= char_scale;
ychar_line3 *= char_scale;

zchar_line1 *= char_scale;
zchar_line2 *= char_scale;
zchar_line3 *= char_scale;

%determine if the rotation is near unity, limit arbitrarily chosen, 0.5 is ok up
%to for example rotation rotx(4)*roty(4)*rotz(4), works for angles < 90 degrees
if( sum(sum(abs(rotation - eye(3)))) < 0.5 )
   %plot characters beside the axes
   %(to avoid altering the volume automatically put up by Gnuplot and with that
   %altering unintentionally the aspect ratio when plotting the first coordinate
   %system and
   %to avoid as much as possible plotting the characters of a subsequent
   %coordinate system on the axes of the first)
   xoffs = [char_disw, 0, char_height; char_disw, 0, char_height];
   yoffs = [0, char_disw, char_height; 0, char_disw, char_height];
   zoffs = [char_width, -char_width, char_dish; char_width, -char_width, char_dish];
else
   %plot characters along the extension of the axes
   xoffs = vectorx(2,:) + vectorx(2,:) / norm( vectorx(2,:) ) * char_disd;
   xoffs = [xoffs; xoffs];
   yoffs = vectory(2,:) + vectory(2,:) / norm( vectory(2,:) ) * char_disd;
   yoffs = [yoffs; yoffs];
   zoffs = vectorz(2,:) + vectorz(2,:) / norm( vectorz(2,:) ) * char_disd;
   zoffs = [zoffs; zoffs];
endif

xchar_line1 += xoffs;
xchar_line2 += xoffs;

ychar_line1 += yoffs;
ychar_line2 += yoffs;
ychar_line3 += yoffs;

zchar_line1 += zoffs;
zchar_line2 += zoffs;
zchar_line3 += zoffs;

if( xy_rot_flag == 1 )
   %apply additional 90 degrees rotation to X and Y, making them visible in views
   %directly on the xy-plane
   xchar_line1 *= rotx( -90 );
   xchar_line2 *= rotx( -90 );

   ychar_line1 *= roty( -90 );
   ychar_line2 *= roty( -90 );
   ychar_line3 *= roty( -90 );
endif

plotpoints( xchar_line1, color )
plotpoints( xchar_line2, color )

plotpoints( ychar_line1, color )
plotpoints( ychar_line2, color )
plotpoints( ychar_line3, color )

plotpoints( zchar_line1, color )
plotpoints( zchar_line2, color )
plotpoints( zchar_line3, color )

if( oldhold == 1 )
   hold( "on" )
else
   hold( "off" )
endif

endfunction
